@@ -113,9 +113,9 @@ long SAML1POSTEncoder::encode(
     GenericResponse& genericResponse,
     XMLObject* xmlObject,
     const char* destination,
-    const EntityDescriptor* recipient,
+    const EntityDescriptor*,
     const char* relayState,
-    const ArtifactGenerator* artifactGenerator,
+    const ArtifactGenerator*,
     const Credential* credential,
     const XMLCh* signatureAlg,
     const XMLCh* digestAlg
