@@ -66,7 +66,7 @@ SAMLArtifactType0002::SAMLArtifactType0002(const string& sourceLocation)
     m_raw+=(char)0x2;
     char buf[HANDLE_LENGTH];
     SAMLConfig::getConfig().generateRandomBytes(buf,HANDLE_LENGTH);
-    for (int i=0; i<HANDLE_LENGTH; i++)
+    for (unsigned int i=0; i<HANDLE_LENGTH; i++)
         m_raw+=buf[i];
     m_raw+=sourceLocation;
 }
