@@ -37,6 +37,7 @@
 #include <xmltooling/logging.h>
 #include <xmltooling/XMLToolingConfig.h>
 #include <xmltooling/io/HTTPRequest.h>
+#include <xmltooling/io/HTTPResponse.h>
 #include <xmltooling/util/NDC.h>
 #include <xmltooling/util/ReplayCache.h>
 
@@ -60,6 +61,7 @@ namespace opensaml {
             xmltooling::XMLObject* decode(
                 std::string& relayState,
                 const GenericRequest& genericRequest,
+                GenericResponse* genericResponse,
                 SecurityPolicy& policy
                 ) const;
         };
@@ -74,6 +76,7 @@ namespace opensaml {
 XMLObject* SAML2ArtifactDecoder::decode(
     string& relayState,
     const GenericRequest& genericRequest,
+    GenericResponse* genericResponse,
     SecurityPolicy& policy
     ) const
 {
@@ -169,6 +172,7 @@ XMLObject* SAML2ArtifactDecoder::decode(
         throw BindingException("ArtifactResponse message did not contain a protocol message.");
     }
     extractMessageDetails(*payload, genericRequest, samlconstants::SAML20P_NS, policy);
+    extractCorrelationID(*httpRequest, dynamic_cast<HTTPResponse*>(genericResponse), relayState, policy);
     policy.evaluate(*payload, &genericRequest);
 
     // Return the payload only.
