@@ -118,7 +118,7 @@ void SAML2MessageDecoder::extractCorrelationID(
 }
 
 void SAML2MessageDecoder::extractMessageDetails(
-    const XMLObject& message, const GenericRequest& request, const XMLCh* protocol, SecurityPolicy& policy
+    const XMLObject& message, const GenericRequest&, const XMLCh* protocol, SecurityPolicy& policy
     ) const
 {
     // Only handle SAML 2.0 messages.
